@@ -1,13 +1,14 @@
 /***************************************************************************
- $RCSfile: job_p.h,v $
+ $RCSfile$
  -------------------
- cvs         : $Id: job_p.h,v 1.8 2005/08/27 06:29:50 aquamaniac Exp $
+ cvs         : $Id$
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
- *          Please see toplevel file COPYING for license details           *
+ * This file is part of the project "AqBanking".                           *
+ * Please see toplevel file COPYING of that project for license details.   *
  ***************************************************************************/
 
 
@@ -18,7 +19,7 @@
 #include "job_l.h"
 #include <gwenhywfar/misc.h>
 #include <gwenhywfar/inherit.h>
-#include <gwenhywfar/misc2.h>
+#include <gwenhywfar/list2.h>
 
 
 struct AB_JOB {
@@ -29,16 +30,18 @@ struct AB_JOB {
   char *resultText;
   AB_JOB_TYPE jobType;
   int availability;
-  GWEN_TYPE_UINT32 jobId;
-  GWEN_TYPE_UINT32 usage;
+  uint32_t jobId;
+  uint32_t usage;
   char *createdBy;
-  GWEN_TYPE_UINT32 idForProvider;
+  uint32_t idForProvider;
   GWEN_DB_NODE *dbData;
   GWEN_TIME *lastStatusChange;
   char *usedTan;
-};
-AB_JOB *AB_Job__freeAll_cb(AB_JOB *j, void *userData);
 
+  AB_TRANSACTION *transaction;
+  AB_TRANSACTION_LIMITS *limits;
+};
+static AB_JOB *AB_Job__clearAll_cb(AB_JOB *j, void *userData);
 
 
 
