@@ -1,13 +1,11 @@
 /***************************************************************************
- $RCSfile: jobgetbalance_l.h,v $
- -------------------
- cvs         : $Id: jobgetbalance_l.h,v 1.1 2004/08/18 16:32:00 aquamaniac Exp $
  begin       : Mon Mar 01 2004
- copyright   : (C) 2004 by Martin Preuss
+ copyright   : (C) 2004-2013 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
- *          Please see toplevel file COPYING for license details           *
+ * This file is part of the project "AqBanking".                           *
+ * Please see toplevel file COPYING of that project for license details.   *
  ***************************************************************************/
 
 
@@ -25,6 +23,9 @@ extern "C" {
 AB_JOB *AB_JobGetBalance_fromDb(AB_ACCOUNT *a, GWEN_DB_NODE *db);
 int AB_JobGetBalance_toDb(const AB_JOB *j, GWEN_DB_NODE *db);
 
+void AB_JobGetBalance_SetAccountStatus(AB_JOB *j,
+                                       const AB_ACCOUNT_STATUS *as);
+
 
 #ifdef __cplusplus
 }
