@@ -1,13 +1,11 @@
 /***************************************************************************
- $RCSfile: jobsingledebitnote_be.h,v $
- -------------------
- cvs         : $Id: jobsingledebitnote_be.h,v 1.3 2005/06/29 03:45:36 aquamaniac Exp $
  begin       : Mon Mar 01 2004
- copyright   : (C) 2004 by Martin Preuss
+ copyright   : (C) 2004-2013 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
- *          Please see toplevel file COPYING for license details           *
+ * This file is part of the project "AqBanking".                           *
+ * Please see toplevel file COPYING of that project for license details.   *
  ***************************************************************************/
 
 
@@ -18,11 +16,28 @@
 #include <aqbanking/jobsingledebitnote.h>
 
 
+/** @addtogroup G_AB_JOBS_XFER_DEBIT
+ *
+ */
+/*@{*/
+
+/** @name Backend Functions
+ *
+ * Functions in this group are only to be called by banking backends.
+ */
+/*@{*/
+
+/**
+ * This function lets the backend specify the limits for some of the fields
+ * of a @ref AB_TRANSACTION.
+ */
+AQBANKING_API AQBANKING_DEPRECATED
 void AB_JobSingleDebitNote_SetFieldLimits(AB_JOB *j,
                                           AB_TRANSACTION_LIMITS *limits);
-AB_JOB *AB_JobSingleDebitNote_fromDb(AB_ACCOUNT *a, GWEN_DB_NODE *db);
-int AB_JobSingleDebitNote_toDb(const AB_JOB *j, GWEN_DB_NODE *db);
+/*@}*/ 
 
 
+/*@}*/ /* addtogroup */
+
 #endif
 
