@@ -27,7 +27,9 @@
 int AH_HHD14_ReadBytesDec(const char *p, int len) {
   int r=0;
   int i;
+  const char *pSave;
 
+  pSave=p;
   for (i=0; i<len; i++) {
     uint8_t c;
 
@@ -35,7 +37,8 @@ int AH_HHD14_ReadBytesDec(const char *p, int len) {
     if (c==0)
       break;
     if (c<'0' || c>'9') {
-      DBG_ERROR(AQHBCI_LOGDOMAIN, "Bad char in data (no decimal digit)");
+      DBG_ERROR(AQHBCI_LOGDOMAIN, "Bad char in data (no decimal digit), pos=%d, byte=%02x", i, c);
+      GWEN_Text_LogString(pSave, len, AQHBCI_LOGDOMAIN, GWEN_LoggerLevel_Error);
       return GWEN_ERROR_INVALID;
     }
     c-='0';
@@ -95,7 +98,12 @@ int AH_HHD14_ExtractDataForLuhnSum(const char *code, GWEN_BUFFER *xbuf) {
   int rv;
   unsigned int len;
   unsigned int i=0;
+  unsigned int LSandFlags;
+  unsigned int numCtrlBytes;
+  unsigned int moreCtrlBytes;
+  unsigned int numBytes;
 
+  /* read LC */
   rv=AH_HHD14_ReadBytesHex(code, 2);
   if (rv<0) {
     DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d) at [%s]", rv, code);
@@ -103,13 +111,42 @@ int AH_HHD14_ExtractDataForLuhnSum(const char *code, GWEN_BUFFER *xbuf) {
   }
   len=((unsigned int) rv);
   code+=2;
+
   if ((strlen(code)+2)<len*2) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "Too few bytes in buffer (%d<%d) at [%s]",
               (int)(strlen(code)+2), len*2, code);
     return GWEN_ERROR_INVALID;
   }
 
-  while(i<len-2) {
+  /* read LS */
+  rv=AH_HHD14_ReadBytesHex(code, 2);
+  if (rv<0) {
+    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d) at [%s]", rv, code);
+    return rv;
+  }
+  code+=2;
+
+  /* add control bytes and start code */
+  LSandFlags = (unsigned int) rv;
+  numCtrlBytes = 0;
+  moreCtrlBytes = LSandFlags & 0x80;
+
+  while (moreCtrlBytes) {
+    rv=AH_HHD14_ReadBytesHex(code+numCtrlBytes*2, 2);                 /* LS */
+    if (rv<0) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d) at [%s]", rv, code);
+      return rv;
+    }
+    numCtrlBytes++;
+    moreCtrlBytes = (unsigned int) rv & 0x80;
+  }
+  numBytes = (LSandFlags & 0x3f) + numCtrlBytes;
+  GWEN_Buffer_AppendBytes(xbuf, code, numBytes*2);
+  code += numBytes*2;
+  i += numBytes + 2;         /* add length of LC and LS */
+
+  /* read LDE1, DE1, LDE2, DE2, ... */
+  while(i<len-1) {
     unsigned int v;
 
     rv=AH_HHD14_ReadBytesHex(code, 2);
@@ -117,8 +154,13 @@ int AH_HHD14_ExtractDataForLuhnSum(const char *code, GWEN_BUFFER *xbuf) {
       DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d) at [%s]", rv, code);
       return rv;
     }
-    v=((unsigned int) rv) & 0xf;
+/*    v=((unsigned int) rv) & 0xf; */
+    v=((unsigned int) rv) & 0x3f; /* as suggested by Martin Kuehn */
     code+=2;
+    if (i+v+1 > len) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "try to read past the end of code (%d) at [%s]", v, code);
+      return GWEN_ERROR_INVALID;
+    }
     GWEN_Buffer_AppendBytes(xbuf, code, v*2);
     code+=v*2;
     i+=v+1;
@@ -221,7 +263,7 @@ void AH_HHD14_ExtractCode(GWEN_BUFFER *cbuf) {
 
 
 int AH_HHD14__Translate(const char *code, GWEN_BUFFER *cbuf) {
-  /*unsigned int totalLength;*/
+  /*unsigned int totalLength;*/ /*TODO: handle total length */
   unsigned int inLenAndFlags;
   unsigned int inLen;
   unsigned int outLenAndFlags;
@@ -257,7 +299,8 @@ int AH_HHD14__Translate(const char *code, GWEN_BUFFER *cbuf) {
   code+=2;
 
   outLen=(inLen+1)/2;
-  snprintf(numbuf, sizeof(numbuf)-1, "%02x", outLen);
+  outLenAndFlags=outLen | (inLenAndFlags & 0x80);
+  snprintf(numbuf, sizeof(numbuf)-1, "%02x", outLenAndFlags);
   numbuf[sizeof(numbuf)-1]=0;
   GWEN_Buffer_AppendString(xbuf, numbuf);
 
@@ -309,7 +352,8 @@ int AH_HHD14__Translate(const char *code, GWEN_BUFFER *cbuf) {
 
       for (i=0; i<inLen; i++) {
         if (code[i]<'0' || code[i]>'9'){
-          /* contains something other than digits, use ascii encoding */
+	  /* contains something other than digits, use ascii encoding */
+	  DBG_ERROR(AQHBCI_LOGDOMAIN, "Switched to ASCII");
           inLenAndFlags|=0x40;
           break;
         }
@@ -321,7 +365,7 @@ int AH_HHD14__Translate(const char *code, GWEN_BUFFER *cbuf) {
       /* ascii */
       //outLen=(inLen+1)/2;
       outLen=inLen;
-      outLenAndFlags=outLen | 0x10; /* add encoding flag to length */
+      outLenAndFlags=outLen | 0x40; /* add encoding flag to length (bit 6) */
       snprintf(numbuf, sizeof(numbuf)-1, "%02x", outLenAndFlags);
       numbuf[sizeof(numbuf)-1]=0;
       GWEN_Buffer_AppendString(xbuf, numbuf);
@@ -424,7 +468,8 @@ int AH_HHD14_Translate(const char *code, GWEN_BUFFER *cbuf) {
 
   rv=AH_HHD14__Translate(GWEN_Buffer_GetStart(xbuf), cbuf);
   if (rv<0) {
-    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "Error translating HHD code (%d)", rv);
+    GWEN_Text_LogString(code, strlen(code), AQHBCI_LOGDOMAIN, GWEN_LoggerLevel_Error);
     GWEN_Buffer_free(xbuf);
     return rv;
   }
