@@ -171,6 +171,10 @@ AH_JOBQUEUE_ADDRESULT AH_JobQueue_AddJob(AH_JOBQUEUE *jq, AH_JOB *j){
     noItan|=(AH_Job_GetFlags(cj) & AH_JOB_FLAGS_NOITAN);
     cj=AH_Job_List_Next(cj);
   } /* while */
+  /* Account for new job when checking limits for thisJobTypeCount and
+   * jobTypeCount */
+  thisJobTypeCount++;
+  GWEN_StringList_AppendString(jobTypes, AH_Job_GetName(j), 0, 1);
   jobTypeCount=GWEN_StringList_Count(jobTypes);
   GWEN_StringList_free(jobTypes);
 
@@ -447,6 +451,7 @@ AH_MSG *AH_JobQueue_ToMessage(AH_JOBQUEUE *jq, AH_DIALOG *dlg){
         AH_Job_SetStatus(j, AH_JobStatusError);
       j=AH_Job_List_Next(j);
     } /* while */
+    AH_Msg_free(msg);
     return 0;
   }
 
@@ -991,6 +996,7 @@ int AH_JobQueue_DispatchMessage(AH_JOBQUEUE *jq,
 
     dbCurr=GWEN_DB_GetNextGroup(dbCurr);
   } /* while */
+  GWEN_DB_Group_free(dbSecurity);
 
   /* set usedTan status accordingly */
   j=AH_Job_List_First(jq->jobs);
