@@ -1,7 +1,4 @@
 /***************************************************************************
- $RCSfile: account_p.h,v $
-                             -------------------
-    cvs         : $Id: account_p.h,v 1.1 2005/08/05 20:33:12 aquamaniac Exp $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -15,31 +12,20 @@
 #define AH_ACCOUNT_P_H
 
 
+#include "hbci_l.h"
 #include "account_l.h"
 
 
-
+typedef struct AH_ACCOUNT AH_ACCOUNT;
 struct AH_ACCOUNT {
-  GWEN_LIST_ELEMENT(AH_ACCOUNT);
-  GWEN_INHERIT_ELEMENT(AH_ACCOUNT);
-
-  GWEN_TYPE_UINT32 usage;
-
-  AH_BANK *bank;
-
-  char *bankId;
-  char *accountId;
-  char *accountName;
-  char *ownerName;
-
-  char *suffix;
-
-  GWEN_STRINGLIST *customers;
-
+  AH_HBCI *hbci;
+  uint32_t flags;
 };
 
+static void GWENHYWFAR_CB AH_Account_freeData(void *bp, void *p);
 
-AH_ACCOUNT *AH_Account__freeAll_cb(AH_ACCOUNT *a, void *userData);
+static void AH_Account_ReadDb(AB_ACCOUNT *a, GWEN_DB_NODE *db);
+static void AH_Account_toDb(AB_ACCOUNT *a, GWEN_DB_NODE *db);
 
 
 #endif /* AH_ACCOUNT_P_H */
