@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: hbci-updates_l.h,v $
+ $RCSfile$
                              -------------------
-    cvs         : $Id: hbci-updates_l.h,v 1.1 2005/08/23 06:25:02 aquamaniac Exp $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -15,10 +15,19 @@
 #define AH_HBCI_UPDATES_L_H
 
 #include "hbci_l.h"
+#include <gwenhywfar/types.h>
 
+/**
+ * This function is called from AH_User_Extend to extend the given user data
+ * DB to the latest version.
+ */
+int AH_HBCI_UpdateDbUser(AH_HBCI *hbci, GWEN_DB_NODE *db);
 
-int AH_HBCI_UpdateDb(AH_HBCI *hbci, GWEN_DB_NODE *db);
-
+/**
+ * This function is called from AH_User_Extend to extend the given account
+ * data DB to the latest version.
+ */
+int AH_HBCI_UpdateDbAccount(AH_HBCI *hbci, GWEN_DB_NODE *db);
 
 
 #endif
