@@ -1,7 +1,4 @@
 /***************************************************************************
- $RCSfile: de_p.h,v $
-                             -------------------
-    cvs         : $Id: de_p.h,v 1.4 2005/04/13 22:18:57 aquamaniac Exp $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -28,18 +25,26 @@ struct AB_BANKINFO_PLUGIN_DE {
 #ifdef HAVE_KTOBLZCHECK
   AccountNumberCheck *checker;
 #endif
-  GWEN_DB_NODE *dbData;
 };
 
+static
+void GWENHYWFAR_CB AB_BankInfoPluginDE_FreeData(void *bp, void *p);
 
-void AB_BankInfoPluginDE_FreeData(void *bp, void *p);
+static
+AB_BANKINFO_PLUGIN *AB_Plugin_BankInfoDE_Factory(GWEN_PLUGIN *pl, AB_BANKING *ab);
 
-AB_BANKINFO_CHECKRESULT
+
+static AB_BANKINFO_CHECKRESULT
 AB_BankInfoPluginDE_CheckAccount(AB_BANKINFO_PLUGIN *bip,
                                  const char *branchId,
                                  const char *bankId,
                                  const char *accountId);
 
+AQBANKING_EXPORT
+GWEN_PLUGIN *bankinfo_de_factory(GWEN_PLUGIN_MANAGER *pm,
+				 const char *name,
+				 const char *fileName);
+
 #endif /* AQBANKING_BANKINFO_DE_P_H */
 
 
