@@ -1,6 +1,6 @@
 /***************************************************************************
  begin       : Fri Apr 02 2004
- copyright   : (C) 2004-2010 by Martin Preuss
+ copyright   : (C) 2004-2015 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -29,6 +29,11 @@
 #include <assert.h>
 
 
+
+/* #define ENABLE_FULL_SEPA_LOG */
+
+
+
 #define CENTURY_CUTOFF_YEAR 79
 
 
@@ -112,9 +117,9 @@ int AHB_SWIFT940_Parse_25(const AHB_SWIFT_TAG *tg,
 
 
 int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
-                          uint32_t flags,
-                          GWEN_DB_NODE *data,
-                          GWEN_DB_NODE *cfg){
+			  uint32_t flags,
+			  GWEN_DB_NODE *data,
+			  GWEN_DB_NODE *cfg){
   const char *p;
   int isStructured;
   int code;
@@ -137,7 +142,6 @@ int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
   }
 
   if (isStructured) {
-#if 1
     AHB_SWIFT_SUBTAG_LIST *stlist;
     int rv;
 
@@ -153,11 +157,15 @@ int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
     else {
       if (code<900) {
 	AHB_SWIFT_SUBTAG *stg;
-	char identifier[6];
-	GWEN_BUFFER *tbuf=NULL;
+	GWEN_DB_NODE *dbSepaTags=NULL;
+	GWEN_BUFFER *bufFullPurpose;
+	int realSepaTagCount=0;
+
+	bufFullPurpose=GWEN_Buffer_new(0, 256, 0, 1);
+
+        /* SEPA transaction, needs special handling */
+        dbSepaTags=GWEN_DB_Group_new("sepa-tags");
 
-	/* SEPA transaction, needs special handling */
-	identifier[0]=0;
 	stg=AHB_SWIFT_SubTag_List_First(stlist);
 	while(stg) {
 	  const char *s;
@@ -188,57 +196,13 @@ int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
 	  case 61:
 	  case 62:
 	  case 63: /* Verwendungszweck */
-	    if (strlen(s)>5 && s[4]=='+') {
-	      if (identifier[0] && tbuf && GWEN_Buffer_GetUsedBytes(tbuf)) {
-		/* close previous identifier */
-		if (strcasecmp(identifier, "EREF+")==0) {
-		  AHB_SWIFT__SetCharValue(data, flags, "endToEndReference", GWEN_Buffer_GetStart(tbuf));
-		}
-		else if (strcasecmp(identifier, "KREF+")==0) {
-		  /* content of PmtInfId */
-		  AHB_SWIFT__SetCharValue(data, flags, "customerReference", GWEN_Buffer_GetStart(tbuf));
-		}
-		else if (strcasecmp(identifier, "MREF+")==0) {
-		  AHB_SWIFT__SetCharValue(data, flags, "mandateReference", GWEN_Buffer_GetStart(tbuf));
-		}
-		else if (strcasecmp(identifier, "CRED+")==0) {
-		  AHB_SWIFT__SetCharValue(data, flags, "creditorIdentifier", GWEN_Buffer_GetStart(tbuf));
-		}
-		else if (strcasecmp(identifier, "DEBT+")==0) {
-		  AHB_SWIFT__SetCharValue(data, flags, "originatorIdentifier", GWEN_Buffer_GetStart(tbuf));
-		}
-		else if (strcasecmp(identifier, "SVWZ+")==0) {
-		  AHB_SWIFT__SetCharValue(data, flags, "purpose", GWEN_Buffer_GetStart(tbuf));
-		}
-		else if (strcasecmp(identifier, "ABWA+")==0) {
-		}
-	      }
-
-	      /* start of an identifier */
-	      strncpy(identifier, s, 5);
-	      identifier[5]=0;
-	      if (tbuf==NULL)
-		tbuf=GWEN_Buffer_new(0, 54, 0, 1);
-	      else
-		GWEN_Buffer_Reset(tbuf);
-	      GWEN_Buffer_AppendString(tbuf, s+5);
-	    }
-	    else {
-	      /* no identifier here */
-	      if (identifier[0] && tbuf && GWEN_Buffer_GetUsedBytes(tbuf)) {
-		/* continuation of previous identifier, add to buffer */
-		if (strcasecmp(identifier, "SVWZ+")==0) {
-		  /* add a space for purpose */
-		  GWEN_Buffer_AppendString(tbuf, " ");
-		}
-		GWEN_Buffer_AppendString(tbuf, s);
-	      }
-	      else {
-		AHB_SWIFT__SetCharValue(data, flags, "purpose", s);
-	      }
-	    }
+	    /* store purpose in any case, if the transaction doesn't have SEPA tags
+	     * we can just use what we produced here */
+	    AHB_SWIFT__SetCharValue(data, flags, "purpose", s);
+	    /* possibly handle full purpose later */
+	    GWEN_Buffer_AppendString(bufFullPurpose, s);
 	    break;
-      
+
 	  case 30: /* BLZ Gegenseite */
 	    AHB_SWIFT__SetCharValue(data, flags, "remoteBankCode", s);
 	    break;
@@ -274,31 +238,181 @@ int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
 	  stg=AHB_SWIFT_SubTag_List_Next(stg);
 	} /* while */
 
-	/* close possibly open identifieres */
-	if (identifier[0] && tbuf && GWEN_Buffer_GetUsedBytes(tbuf)) {
-	  /* close previous identifier */
-	  if (strcasecmp(identifier, "EREF+")==0) {
-	    AHB_SWIFT__SetCharValue(data, flags, "endToEndReference", GWEN_Buffer_GetStart(tbuf));
-	  }
-	  else if (strcasecmp(identifier, "KREF+")==0) {
-	    AHB_SWIFT__SetCharValue(data, flags, "customerReference", GWEN_Buffer_GetStart(tbuf));
-	  }
-	  else if (strcasecmp(identifier, "MREF+")==0) {
-	    AHB_SWIFT__SetCharValue(data, flags, "mandateReference", GWEN_Buffer_GetStart(tbuf));
-	  }
-	  else if (strcasecmp(identifier, "CRED+")==0) {
-	    AHB_SWIFT__SetCharValue(data, flags, "creditorIdentifier", GWEN_Buffer_GetStart(tbuf));
-	  }
-	  else if (strcasecmp(identifier, "DEBT+")==0) {
-	    AHB_SWIFT__SetCharValue(data, flags, "originatorIdentifier", GWEN_Buffer_GetStart(tbuf));
-	  }
-	  else if (strcasecmp(identifier, "SVWZ+")==0) {
-	    AHB_SWIFT__SetCharValue(data, flags, "purpose", GWEN_Buffer_GetStart(tbuf));
-	  }
-	  else if (strcasecmp(identifier, "ABWA+")==0) {
+	if (GWEN_Buffer_GetUsedBytes(bufFullPurpose)) {
+	  const char *s;
+	  const char *sLastTagStart;
+
+#ifdef ENABLE_FULL_SEPA_LOG
+	  DBG_ERROR(0, "FullPurposeBuffer:");
+	  GWEN_Buffer_Dump(bufFullPurpose, 2);
+#endif
+
+	  s=GWEN_Buffer_GetStart(bufFullPurpose);
+	  sLastTagStart=s;
+
+	  /* sample all SEPA fields from concatenated string of purpose lines */
+	  while(*s) {
+	    /* look for begin of next tag */
+	    while(*s) {
+	      if ((*s && isalpha(*s)) &&
+		  (s[1] && isalpha(s[1])) &&
+		  (s[2] && isalpha(s[2])) &&
+		  (s[3] && isalpha(s[3])) &&
+		  s[4]=='+')
+		break;
+	      /* not the beginning of a SEPA field, just skip */
+	      s++;
+	    }
+
+	    /* found begin of the next SEPA field or end of buffer */
+	    if (s > sLastTagStart) {
+	      int tagLen;
+
+	      /* we currently have a field, close that first */
+	      tagLen=s-sLastTagStart;
+
+#ifdef ENABLE_FULL_SEPA_LOG
+	      DBG_ERROR(0, "Current tag:");
+	      GWEN_Text_LogString(sLastTagStart, tagLen, 0, GWEN_LoggerLevel_Error);
+#endif
+
+	      /* check tag length (must be long enough for 'XXX+', i.e. at least 5 bytes) */
+	      if (tagLen>5 && sLastTagStart[4]=='+') {
+		char sIdentifier[6];
+		const char *sPayload;
+
+		/* ok, 5 bytes or more, 4 alphas and a plus sign, should be the begin of a SEPA tag */
+		strncpy(sIdentifier, sLastTagStart, 5);
+		sIdentifier[5]=0;
+
+		/* remove leading blanks */
+		sPayload=sLastTagStart+5;
+		tagLen-=5;
+		while(tagLen>0 && *sPayload && isblank(*sPayload)) {
+		  sPayload++;
+		  tagLen--;
+		}
+
+		/* remove trailing blanks */
+		if (tagLen>0) {
+		  while(tagLen>0) {
+		    if (!isblank(sPayload[tagLen-1]))
+		      break;
+		    tagLen--;
+		  }
+		}
+
+		/* store tag, if still data left */
+		if (tagLen>0) {
+		  char *sCopyPayload;
+
+		  sCopyPayload=strndup(sPayload, tagLen);
+		  GWEN_DB_SetCharValue(dbSepaTags, GWEN_DB_FLAGS_DEFAULT, sIdentifier, sCopyPayload);
+		  free(sCopyPayload);
+		  realSepaTagCount++;
+		}
+		else {
+		  DBG_WARN(GWEN_LOGDOMAIN, "Ignoring empty SEPA field \"%s\"", sIdentifier);
+		}
+	      }
+	      else {
+		/* tag is shorter than 5 bytes or pos 4 doesn't contain a plus, treat as normal purpose */
+                if (tagLen>0) {
+		  char *sCopyPayload;
+
+		  sCopyPayload=strndup(sLastTagStart, tagLen);
+                  GWEN_DB_SetCharValue(dbSepaTags, GWEN_DB_FLAGS_DEFAULT, "_purpose", sCopyPayload);
+		  free(sCopyPayload);
+                }
+              }
+	    }
+
+	    if (*s) {
+	      /* save start of next tag */
+	      sLastTagStart=s;
+	      /* skip XXX+ at the beginning, otherwise we would immediately stop in next loop
+	       * we know that the next 5 bytes are valid, so it is safe to skip them */
+	      s+=5;
+	    }
+	  } /* while */
+
+	} /* if GWEN_Buffer_GetUsedBytes(bufFullPurpose) */
+	/* buffer no longer needed */
+	GWEN_Buffer_free(bufFullPurpose);
+
+	/* handle SEPA tags, iterate through them and parse according to variable name */
+	if (realSepaTagCount>0 && GWEN_DB_Variables_Count(dbSepaTags)) {
+	  GWEN_DB_NODE *dbVar;
+
+#ifdef ENABLE_FULL_SEPA_LOG
+	  DBG_ERROR(0, "Got these SEPA tags:");
+	  GWEN_DB_Dump(dbSepaTags, 2);
+#endif
+
+          /* clear purpose variable, since we are about to add it back from SEPA tags */
+          GWEN_DB_DeleteVar(data, "purpose");
+
+	  dbVar=GWEN_DB_GetFirstVar(dbSepaTags);
+	  while(dbVar) {
+	    const char *sVarName;
+
+	    sVarName=GWEN_DB_VariableName(dbVar);
+	    if (sVarName && *sVarName) {
+	      GWEN_BUFFER *tbuf;
+	      GWEN_DB_NODE *dbValue;
+
+	      /* sample all values into a buffer and concatenate */
+	      tbuf=GWEN_Buffer_new(0, 128, 0, 1);
+	      dbValue=GWEN_DB_GetFirstValue(dbVar);
+	      while(dbValue) {
+		const char *s;
+
+		s=GWEN_DB_GetCharValueFromNode(dbValue);
+		if (s && *s)
+		  GWEN_Buffer_AppendString(tbuf, s);
+
+		dbValue=GWEN_DB_GetNextValue(dbValue);
+	      }
+
+	      if (strcasecmp(sVarName, "EREF+")==0) {
+		AHB_SWIFT__SetCharValue(data, flags, "endToEndReference", GWEN_Buffer_GetStart(tbuf));
+	      }
+	      else if (strcasecmp(sVarName, "KREF+")==0) {
+		AHB_SWIFT__SetCharValue(data, flags, "customerReference", GWEN_Buffer_GetStart(tbuf));
+	      }
+	      else if (strcasecmp(sVarName, "MREF+")==0) {
+		AHB_SWIFT__SetCharValue(data, flags, "mandateId", GWEN_Buffer_GetStart(tbuf));
+	      }
+	      else if (strcasecmp(sVarName, "CRED+")==0) {
+		AHB_SWIFT__SetCharValue(data, flags, "creditorSchemeId", GWEN_Buffer_GetStart(tbuf));
+	      }
+	      else if (strcasecmp(sVarName, "DEBT+")==0) {
+		AHB_SWIFT__SetCharValue(data, flags, "originatorIdentifier", GWEN_Buffer_GetStart(tbuf));
+	      }
+	      else if (strcasecmp(sVarName, "SVWZ+")==0) {
+		AHB_SWIFT__SetCharValue(data, flags, "purpose", GWEN_Buffer_GetStart(tbuf));
+	      }
+	      else if (strcasecmp(sVarName, "ABWA+")==0) {
+              }
+              else if (strcasecmp(sVarName, "_purpose")==0) {
+                /* manually added tag (i.e. data outside a tag)
+		 * TODO: only add if there was no real purpose field (i.e. no "SVWZ+") */
+		AHB_SWIFT__SetCharValue(data, flags, "purpose", GWEN_Buffer_GetStart(tbuf));
+	      }
+
+              GWEN_Buffer_free(tbuf);
+            }
+
+	    dbVar=GWEN_DB_GetNextVar(dbVar);
 	  }
 	}
-	GWEN_Buffer_free(tbuf);
+
+#ifdef ENABLE_FULL_SEPA_LOG
+	DBG_ERROR(0, "SEPA Tags:");
+	GWEN_DB_Dump(dbSepaTags, 2);
+#endif
+
+	GWEN_DB_Group_free(dbSepaTags);
       } /* if sepa */
       else {
 	AHB_SWIFT_SUBTAG *stg;
@@ -374,123 +488,8 @@ int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
       }
     } /* if really structured */
     AHB_SWIFT_SubTag_List_free(stlist);
-#else
-    const char *p2;
-    char *s;
-    int id;
-
-    /* store code */
-    GWEN_DB_SetIntValue(data, flags, "textkey", code);
-
-    while(*p) {
-      int intVal;
-
-      if (strlen(p)<3) {
-	DBG_ERROR(AQBANKING_LOGDOMAIN, "Bad field in :86: tag (%s)", p);
-	GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
-                              "SWIFT: Bad field in :86: tag");
-        return -1;
-      }
-      p++; /* skip '?' */
-      /* read field id */
-      if (*p==10)
-        p++;
-      if (!*p) {
-	DBG_ERROR(AQBANKING_LOGDOMAIN, "Partial field id");
-        GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
-                              "SWIFT: Partial field id");
-        return -1;
-      }
-      id=((*p-'0')*10);
-      p++;
-
-      if (*p==10)
-        p++;
-      if (!*p) {
-        DBG_ERROR(AQBANKING_LOGDOMAIN, "Partial field id");
-        GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error,
-                              "SWIFT: Partial field id");
-        return -1;
-      }
-      id+=(*p-'0');
-      p++;
-
-      p2=p;
-
-      /* find end of field/beginning of next field */
-      while(*p2 && *p2!='?') p2++;
-      s=(char*)GWEN_Memory_malloc(p2-p+1);
-      memmove(s, p, p2-p+1);
-      s[p2-p]=0;
-      AHB_SWIFT_Condense(s, keepMultipleBlanks);
-      DBG_DEBUG(AQBANKING_LOGDOMAIN, "Current field is %02d (%s)", id, s);
-      /* now id is the field id, s points to the field content */
-      if (*s) {
-        switch(id) {
-        case 0: /* Buchungstext */
-          AHB_SWIFT__SetCharValue(data, flags, "transactionText", s);
-          break;
-        case 10: /* Primanota */
-          AHB_SWIFT__SetCharValue(data, flags, "primanota", s);
-          break;
-    
-        case 20:
-        case 21:
-        case 22:
-        case 23:
-        case 24:
-        case 25:
-        case 26:
-        case 27:
-        case 28:
-        case 29:
-        case 60:
-        case 61:
-        case 62:
-        case 63: /* Verwendungszweck */
-          AHB_SWIFT__SetCharValue(data, flags, "purpose", s);
-          break;
-    
-        case 30: /* BLZ Gegenseite */
-          AHB_SWIFT__SetCharValue(data, flags, "remoteBankCode", s);
-          break;
-    
-        case 31: /* Kontonummer Gegenseite */
-          AHB_SWIFT__SetCharValue(data, flags, "remoteAccountNumber", s);
-          break;
-    
-        case 32: 
-        case 33: /* Name Auftraggeber */
-          AHB_SWIFT__SetCharValue(data, flags, "remoteName", s);
-          break;
-    
-	case 34: /* Textschluesselergaenzung */
-	  if (1==sscanf(s, "%d", &intVal)) {
-	    GWEN_DB_SetIntValue(data, flags, "textkeyExt", intVal);
-	  }
-	  else {
-	    DBG_WARN(AQBANKING_LOGDOMAIN,
-                     "Value [%s] is not a number (textkeyext)", s);
-	  }
-	  break;
-
-	case 38: /* IBAN */
-	  AHB_SWIFT__SetCharValue(data, flags, "remoteIban", s);
-          break;
-
-        default: /* ignore all other fields (if any) */
-          DBG_WARN(AQBANKING_LOGDOMAIN, "Unknown :86: field \"%02d\" (%s) (%s)", id, s,
-                   AHB_SWIFT_Tag_GetData(tg));
-          break;
-        } /* switch */
-      }
-      p=p2;
-      GWEN_Memory_dealloc(s);
-    } /* while */
-#endif
-  } /* if structured */
-
-  if (!isStructured) {
+  } /* if isStructured */
+  else {
     char *pcopy=strdup(p);
     char *p1;
 
@@ -504,6 +503,9 @@ int AHB_SWIFT940_Parse_86(const AHB_SWIFT_TAG *tg,
 	*p2=0;
 	p2++;
       }
+
+      /* look for pattern "KTO/BLZ", if found try to extract remote account info
+       * from unstructured purpose string */
       if (-1!=GWEN_Text_ComparePattern(p1, "*KTO/BLZ */*", 0)) {
 	char *p3;
 	char *kto;
@@ -1099,7 +1101,10 @@ int AHB_SWIFT940_Parse_NS(const AHB_SWIFT_TAG *tg,
 }
 
 
-
+/* Import SWIFT MT940 data.
+   @param tl input: list of tags. Tags are lines in a SWIFT data block (block 4). A tag has an
+          id and content. See the AHB_SWIFT_Tag_new function for more information.
+ */
 int AHB_SWIFT940_Import(AHB_SWIFT_TAG_LIST *tl,
 			GWEN_DB_NODE *data,
 			GWEN_DB_NODE *cfg,
@@ -1289,6 +1294,18 @@ int AHB_SWIFT940_Import(AHB_SWIFT_TAG_LIST *tl,
 	    }
 	  }
 	}
+	else if (strcmp(id, "21")==0) {
+	  const char *p;
+
+	  p=AHB_SWIFT_Tag_GetData(tg);
+	  assert (p);
+	  if (0==strcmp(p, "NONREF")) {
+	    DBG_INFO(AQBANKING_LOGDOMAIN, "Ignoring related reference '%s' in document tag 21.", p);
+	  }
+	  else {
+	    DBG_WARN(AQBANKING_LOGDOMAIN, "Unexpected related reference '%s' in document tag 21 encountered.", p);
+	  }
+	}
 	else {
 	  DBG_WARN(AQBANKING_LOGDOMAIN, "Unexpected tag '%s' found.", id)
 	  DBG_WARN(AQBANKING_LOGDOMAIN, "To debug set environment variable AQBANKING_LOGLEVEL=info and rerun");
